"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
exports.MockLink = void 0;
var client_1 = require("@apollo/client");
var graphql_1 = require("graphql");
var MockLink = /** @class */ (function (_super) {
    __extends(MockLink, _super);
    function MockLink(mock) {
        var _this = _super.call(this) || this;
        _this.mock = mock;
        return _this;
    }
    MockLink.prototype.updateMock = function (mock) {
        this.mock = mock;
    };
    MockLink.prototype.request = function (operation) {
        var _this = this;
        return new client_1.Observable(function (obs) {
            var mock = _this.mock;
            var _a = operation.operationName, operationName = _a === void 0 ? '' : _a;
            var response = null;
            if (typeof mock === 'function') {
                response = mock(operation);
            }
            else {
                var mockForOperation = mock[operationName || ''];
                response =
                    typeof mockForOperation === 'function'
                        ? mockForOperation(operation)
                        : mockForOperation;
            }
            var result;
            if (response == null) {
                var message = "Can\u2019t perform GraphQL operation '" + operationName + "' because no valid mocks were found";
                if (typeof mock === 'object') {
                    var operationNames = Object.keys(mock);
                    // We will provide a more helpful message when it looks like they just provided data,
                    // not an object mapping names to fixtures.
                    var looksLikeDataNotFixtures = operationNames.every(function (name) { return name === name.toLowerCase(); });
                    message += looksLikeDataNotFixtures
                        ? " (it looks like you tried to provide data directly to the mock GraphQL client. You need to provide your fixture on the key that matches its operation name. To fix this, simply change your code to read 'mockGraphQLClient({" + operationName + ": yourFixture}).'"
                        : " (you provided an object that had mocks only for the following operations: " + Object.keys(mock).join(', ') + ").";
                }
                else {
                    message +=
                        ' (you provided a function that did not return a valid mock result)';
                }
                var error = new Error(message);
                result = error;
            }
            else if (response instanceof graphql_1.GraphQLError) {
                result = {
                    errors: [response]
                };
            }
            else if (response instanceof Error) {
                result = { errors: [new graphql_1.GraphQLError(response.message)] };
            }
            else {
                result = {
                    data: response
                };
            }
            obs.next(result);
            obs.complete();
        });
    };
    return MockLink;
}(client_1.ApolloLink));
exports.MockLink = MockLink;
