"use strict";
exports.__esModule = true;
exports.toHavePerformedGraphQLOperation = void 0;
var jest_matcher_utils_1 = require("jest-matcher-utils");
var utilities_1 = require("../utilities");
var utilities_2 = require("./utilities");
function toHavePerformedGraphQLOperation(graphQL, document, variables) {
    var _this = this;
    utilities_2.assertIsGraphQL(graphQL, {
        expectation: 'toHavePerformedGraphQLOperation',
        isNot: this.isNot
    });
    var foundByOperation = graphQL.operations.all({
        query: document
    });
    var foundByVariables = variables == null
        ? foundByOperation
        : foundByOperation.filter(function (operation) {
            return Object.keys(variables).every(function (key) {
                return _this.equals(variables[key], operation.variables[key]);
            });
        });
    var pass = foundByVariables.length > 0;
    var type = utilities_1.operationTypeFromDocument(document) || 'query';
    var name = utilities_1.operationNameFromDocument(document) || 'unnamed';
    var message = pass
        ? function () {
            return jest_matcher_utils_1.matcherHint('.not.toHavePerformedGraphQLOperation') + "\n\n" +
                ("Expected not to have performed GraphQL " + type + ":\n  " + jest_matcher_utils_1.EXPECTED_COLOR(name) + "\n" + (variables
                    ? "With variables matching:\n  " + jest_matcher_utils_1.printExpected(variables) + "\n"
                    : '')) +
                ("But " + foundByVariables.length + " matching " + (foundByVariables.length === 1 ? 'operation was' : 'operations were') + " found.\n");
        }
        : function () {
            return "" + (jest_matcher_utils_1.matcherHint('.toHavePerformedGraphQLOperation') + "\n\n" +
                ("Expected to have performed GraphQL " + type + ":\n  " + jest_matcher_utils_1.EXPECTED_COLOR(name) + "\n" + (variables
                    ? "With props matching:\n  " + jest_matcher_utils_1.printExpected(variables) + "\n"
                    : ''))) + (foundByVariables.length === 0
                ? "But no matching operations were found.\n"
                : "But the " + (foundByVariables.length === 1
                    ? 'found operation has'
                    : 'found operations have') + " the following variable differences:\n\n" + diffs(foundByVariables, variables, _this.expand));
        };
    return { pass: pass, message: message };
}
exports.toHavePerformedGraphQLOperation = toHavePerformedGraphQLOperation;
function diffs(operations, variables, expand) {
    return operations.reduce(function (diffs, operation, index) {
        return "" + diffs + (index === 0 ? '' : '\n\n') + normalizedDiff(operation, variables, {
            expand: expand,
            showLegend: index === 0
        });
    }, '');
}
function normalizedDiff(operation, variables, _a) {
    var _b = _a.expand, expand = _b === void 0 ? false : _b, _c = _a.showLegend, showLegend = _c === void 0 ? false : _c;
    var result = utilities_2.diffVariablesForOperation(operation, variables, {
        expand: expand
    }) || '';
    return showLegend ? result : result.split('\n\n')[1];
}
